package ij.plugin.filter;

import ij.IJ;
import ij.ImagePlus;
import ij.WindowManager;
import ij.gui.GenericDialog;
import ij.gui.Roi;
import ij.measure.Calibration;
import ij.process.ImageProcessor;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.Image;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.awt.print.Pageable;
import java.awt.print.Paper;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.ServiceUI;
import javax.print.DocFlavor;
import javax.print.PrintService;
import sun.print.ServiceDialog;
import java.util.ArrayList;

/** This plugin implements the File/Page Setup and File/Print commands. */
public class Printer implements PlugInFilter, Printable, Pageable {
        private ImagePlus imp;
        private static double scaling = 100.0;
        private static boolean drawBorder;
        private static boolean center = true;
        private static boolean label;
        private static boolean printSelection;
        private static boolean rotate;
        private static int fontSize = 12;
        protected PageFormat pageFormat;      // Paper plus page orientation
        protected double scalefactor;     // How much to scale before printing
        protected int numPages;           // How many pages in the document
        protected double printX, printY;  // coordinates of upper-left of print area
        protected double printWidth;      // Width of the printable area
        protected double printHeight;     // Height of the printable area

            // For a document of n pages, this list stores the lengths of pages
        // 0 through n-2.  The last page is assumed to have a full length
        ArrayList pageLengths = new ArrayList();
        /** Return the number of pages. This is a Pageable method.   */
        public int getNumberOfPages() { return numPages; }
        /**
         * This Pageable method returns the Printable object for the specified
         * page.  Since this class implements both Pageable and Printable, it just
         * returns this.
         **/
        public Printable getPrintable(int pagenum) { return this; }
        /**
         * Return the PageFormat object for the specified page.  This is a
         * Pageable method. This implementation uses the computed length of the
         * page in the returned PageFormat object.  The PrinterJob will use this
         * as a clipping region, which will prevent extraneous parts of the
         * document from being drawn in the top and bottom margins.
         **/
        public PageFormat getPageFormat(int pagenum) {
            // On the last page, just return the user-specified page format
            if (pagenum == numPages-1) return pageFormat;

            // Otherwise, look up the height of this page and return an
            // appropriate PageFormat.
            double pageLength = ((Double)pageLengths.get(pagenum)).doubleValue();
            PageFormat f = (PageFormat) pageFormat.clone();
            Paper p = f.getPaper();
            if (f.getOrientation() == PageFormat.PORTRAIT)
                p.setImageableArea(printX*scalefactor, printY*scalefactor,
                                   printWidth*scalefactor, pageLength*scalefactor);
            else
                p.setImageableArea(printY*scalefactor, printX*scalefactor,
                                   pageLength*scalefactor, printWidth*scalefactor);
            f.setPaper(p);
            return f;
        }

        public int setup(String arg, ImagePlus imp) {
                if (!IJ.isJava14())
                        {IJ.error("Print", "Printing requires Java 1.2 or later"); return DONE;}
                if (arg.equals("setup"))
                        {pageSetup(); return DONE;}
                this.imp = imp;
                IJ.register(Printer.class);
                return DOES_ALL+NO_CHANGES;
        }

        public void run(ImageProcessor ip) {
                //pageSetup();
                print(imp);
        }

        void pageSetup() {
                GenericDialog gd = new GenericDialog("Page Setup");
                gd.addNumericField("Scale:", scaling, 0, 3, "%");
                gd.addCheckbox("Draw Border", drawBorder);
                gd.addCheckbox("Center on Page", center);
                gd.addCheckbox("Print Title", label);
                gd.addCheckbox("Selection Only", printSelection);
                gd.addCheckbox("Rotate 90"+IJ.degreeSymbol, rotate);
                gd.showDialog();
                if (gd.wasCanceled())
                        return;
                scaling = gd.getNextNumber();
                if (scaling<5.0) scaling = 5;
                drawBorder = gd.getNextBoolean();
                center = gd.getNextBoolean();
                label = gd.getNextBoolean();
                printSelection = gd.getNextBoolean();
                rotate = gd.getNextBoolean();
        }

        void print(ImagePlus imp) {
            boolean pDialogState = false;
            PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();
                PrinterJob pj = PrinterJob.getPrinterJob();
                /*PrintService ps = pj.getPrintService();
                DocFlavor df = ps.getSupportedDocFlavors()[0];
                GraphicsConfiguration gc = null;
                ServiceDialog sd = new ServiceDialog(gc, 0, 0, ps, df,aset, new ij.plugin.frame.PlugInFrame("Servive Dialog") );
                sd.setVisible(true);*/
                pageFormat = pj.defaultPage();
                pj.setPrintable(this);
                        try {
                                pDialogState = pj.printDialog(aset);
                            if (IJ.macroRunning() || pDialogState)
                                    imp.startTiming();
                                    if (pDialogState) {
                                           pj.print(aset);
                            }

                        } catch (java.security.AccessControlException ace) {
                            IJ.log(""+ ace);
                        }
                        catch (PrinterException e) {
                                IJ.log(""+e);
                        }
        }

        public int print(Graphics g, PageFormat pf, int pageIndex) {
                if (pageIndex != 0) return NO_SUCH_PAGE;
                ImageProcessor ip = imp.getProcessor();
                Roi roi = imp.getRoi();
                if (printSelection && roi!=null && roi.isArea() )
                        ip = ip.crop();
                if (rotate)
                        ip = ip.rotateLeft();
                //new ImagePlus("ip", ip.duplicate()).setVisible(true);
                int width = ip.getWidth();
                int height = ip.getHeight();
                int margin = 0;
                if (drawBorder) margin = 1;
                double scale = scaling/100.0;
                int dstWidth = (int)(width*scale);
                int dstHeight = (int)(height*scale);
                int pageX = (int)pf.getImageableX();
                int pageY = (int)pf.getImageableY();
                int dstX = pageX+margin;
                int dstY = pageY+margin;
                Image img = ip.createImage();
                double pageWidth = pf.getImageableWidth()-2*margin;
                double pageHeight = pf.getImageableHeight()-2*margin;
                if (label && pageWidth-dstWidth<fontSize+5) {
                        dstY += fontSize+5;
                        pageHeight -= fontSize+5;
                }
                if (dstWidth>pageWidth || dstHeight>pageHeight) {
                        // scale to fit page
                        double hscale = pageWidth/dstWidth;
                        double vscale = pageHeight/dstHeight;
                        double scale2 = hscale<=vscale?hscale:vscale;
                        dstWidth = (int)(dstWidth*scale2);
                        dstHeight = (int)(dstHeight*scale2);
                } else if (center) {
                        dstX += (pageWidth-dstWidth)/2;
                        dstY += (pageHeight-dstHeight)/2;
                }
                g.drawImage(img,
                        dstX, dstY, dstX+dstWidth, dstY+dstHeight,
                        0, 0, width, height,
                        null);
                if (drawBorder)
                        g.drawRect(dstX-1, dstY-1, dstWidth+1, dstHeight+1);
                if (label) {
                        g.setFont(new Font("SanSerif", Font.PLAIN, fontSize));
                        g.setColor(Color.black);
                        g.drawString(imp.getTitle(), pageX+5, pageY+fontSize);
                }
                return PAGE_EXISTS;
        }

}
